from jsonpath_ng import parse
from .paginate import ec2_paginate

#####################################

class describe_regions(ec2_paginate):
    def name(self): return __class__.__name__        
    def jsonpath_expression(self): return parse('Regions[*]')

#####################################

class describe_instances(ec2_paginate):
    def name(self): return __class__.__name__        
    def jsonpath_expression(self): return parse('Reservations[*].Instances[*]')

#####################################

class describe_vpc_endpoints(ec2_paginate):
    def name(self): return __class__.__name__
    def jsonpath_expression(self): return parse('VpcEndpoints[*]')

#####################################

class describe_nat_gateways(ec2_paginate):
    def name(self): return __class__.__name__
    def jsonpath_expression(self): return parse('NatGateways[*]')

#####################################

class describe_subnets(ec2_paginate):
    def name(self): return __class__.__name__
    def jsonpath_expression(self): return parse('Subnets[*]')

#####################################

class describe_route_tables(ec2_paginate):
    def name(self): return __class__.__name__
    def jsonpath_expression(self): return parse('RouteTables[*]')

#####################################

class describe_prefix_lists(ec2_paginate):
    def name(self): return __class__.__name__
    def jsonpath_expression(self): return parse('PrefixLists[*]')

#####################################

class describe_vpcs(ec2_paginate):
    def name(self): return __class__.__name__
    def jsonpath_expression(self): return parse('Vpcs[*]')

#####################################

class describe_instance_types(ec2_paginate):
    def name(self): return __class__.__name__
    def jsonpath_expression(self): return parse('InstanceTypes[*]')

#####################################

class describe_launch_templates(ec2_paginate):
    def name(self): return __class__.__name__
    def jsonpath_expression(self): return parse('LaunchTemplates[*]')

#####################################

class describe_launch_template_versions(ec2_paginate):
    def name(self): return __class__.__name__
    def jsonpath_expression(self): return parse('LaunchTemplateVersions[*]')



import os
import asyncio
from functools import wraps, partial
from concurrent.futures import ThreadPoolExecutor

MAX_WORKERS = int(os.environ.get('MAX_WORKERS', '16'))

# Worker pool
worker_pool = ThreadPoolExecutor(max_workers=MAX_WORKERS)

def tasks(s, jdata, func, output):
    return [run_analysis(s[i.get("account_id")][i.get("region")],i, func, output) for i in jdata]

def worker(func):
    """
    Runs blocking code asynchronously in worker thread from worker pool
    """
    @wraps(func)
    async def run(*args, **kwargs):
        pfunc = partial(func, *args, **kwargs)
        loop = asyncio.get_event_loop()
        return await loop.run_in_executor(worker_pool, pfunc)
    return run

@worker
def run_analysis(s, record, func, output):
    if s: output.append(func(s,record))



from jsonpath_ng import parse
from .paginate import cw_paginate

from datetime import datetime as dt
from datetime import timedelta as td

#####################################

class get_metric_data(cw_paginate):
    def name(self): return __class__.__name__        
    def jsonpath_expression(self): return parse('MetricDataResults[*]')

#####################################

def _get_metric_stat(**kargs):

  stat = kargs.get('Stat')

  try:
      md = kargs.get('session').client('cloudwatch').get_metric_statistics(
                                                        StartTime=dt.now() - td(days=kargs.get('days')), 
                                                        EndTime=dt.now(), 
                                                        Namespace=f"AWS/{kargs.get('Namespace')}",
                                                        MetricName=kargs.get('MetricName'),
                                                        Dimensions=[kargs.get('Dimensions')],
                                                        Period=kargs.get('Period'), 
                                                        Statistics=[stat])
      return md['Datapoints'][0][stat] if len(md['Datapoints']) > 0 else None
  except Exception as e:
        print(e)
        return None

#####################################
# Utility
#####################################

def _get_metric_data(**kargs):

    try:
        return get_metric_data(kargs.get('session'), 
            StartTime=dt.now()-td(days=kargs.get('days')), 
            EndTime=dt.now(), 
            MetricDataQueries=kargs.get('MetricDataQueries'),
            fields=kargs.get('fields',[]));
    except Exception as e:
        print(e)
        return None
    
#####################################

def _get_metric_data_queries(**kargs):
    dims = kargs.get('Dimensions')
    return [{
         "Id": f"{metric}_{dims['Value']}", 
         "MetricStat": {
             "Metric": {
                 "Namespace": f"AWS/{kargs.get('Namespace')}",
                 "MetricName": metric,
                 "Dimensions": [dims]
             },
             "Period": kargs.get('Period'), 
             "Stat": kargs.get('Stat'),
             "Unit": kargs.get("Unit"),
         },
         'Label': metric
      } for metric in kargs.get('MetricNames')]
